/*
 * Linux_BIOSProduct.c
 *
 * (C) Copyright IBM Corp. 2003
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://oss.software.ibm.com/developerworks/opensource/license-cpl.html
 *
 * Author:       Viktor Mihajlovski <mihajlov@de.ibm.com>
 * Contributors: James Jeffers <jamesjef@us.ibm.com>
 *
 * Interface Type : Common Manageability Programming Interface ( CMPI )
 *
 * Description: The Product Class. Anchor for BIOS Feature.
 * 
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "cmpidt.h"
#include "cmpift.h"
#include "cmpimacs.h"

#include "dmiinfo.h"

unsigned char CMPI_true=1;
unsigned char CMPI_false=0;

CMPIBroker * _broker;

/* ---------------------------------------------------------------------------*/
/* private declarations                                                       */
#ifdef DEBUG
    int _debug = 1;
#else
    int _debug = 0;
#endif

static char * _ClassName     = "Linux_BIOSProduct";

static CMPIObjectPath *makeObjectPath(DMI_BIOSPRODUCT *bp,CMPIObjectPath *op);
static CMPIInstance *makeInstance(DMI_BIOSPRODUCT *bp,CMPIObjectPath *op);

/* ---------------------------------------------------------------------------*/
/*                      Instance Provider Interface                           */
/* ---------------------------------------------------------------------------*/


CMPIStatus Linux_BIOSProductCleanup( CMPIInstanceMI * mi, 
  CMPIContext * ctx) {
  cimdmi_term();
  CMReturn(CMPI_RC_OK);
}

CMPIStatus Linux_BIOSProductEnumInstanceNames( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * ref) { 
  CMPIStatus rc = {CMPI_RC_OK, NULL};
  CMPIObjectPath *op;
  DMI_BIOSPRODUCT *biosprod;

  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI EnumInstanceNames()\n", _ClassName );
  
  biosprod=cimdmi_getBiosProduct();
  if (biosprod) {
    op = makeObjectPath(biosprod,ref);
    if (op) {
      CMReturnObjectPath(rslt,op);
    } else {
      rc.rc=CMPI_RC_ERR_NOT_FOUND;
      rc.msg=CMNewString(_broker,
			 "Failed to extract BIOS Product Information",NULL);
    }
  }else {
    rc.rc=CMPI_RC_ERROR;
    rc.msg=CMNewString(_broker,"Failed to locate BIOS",NULL);
  }

  CMReturnDone( rslt );
  return rc;;
}

CMPIStatus Linux_BIOSProductEnumInstances( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * ref, 
  char ** properties) { 
  CMPIStatus rc = {CMPI_RC_OK, NULL};
  CMPIInstance *inst;
  DMI_BIOSPRODUCT *biosprod;

  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI EnumInstances()\n", _ClassName );

  biosprod=cimdmi_getBiosProduct();
  if (biosprod) {
    inst = makeInstance(biosprod,ref);
    if (inst) {
      CMReturnInstance(rslt,inst);
    } else {
      rc.rc=CMPI_RC_ERR_NOT_FOUND;
      rc.msg=CMNewString(_broker,
			 "Failed to extract BIOS Product Information",NULL);
    }
  }else {
    rc.rc=CMPI_RC_ERROR;
    rc.msg=CMNewString(_broker,"Failed to locate BIOS",NULL);
  }

  CMReturnDone( rslt );
  return rc;
}

CMPIStatus Linux_BIOSProductGetInstance( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * cop, 
  char ** properties) {
  CMPIStatus rc = {CMPI_RC_OK, NULL};
  CMPIInstance *inst;
  DMI_BIOSPRODUCT *biosprod;

  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI GetInstance()\n", _ClassName );

  biosprod=cimdmi_getBiosProduct();
  if (biosprod) {
    inst = makeInstance(biosprod,cop);
    if (inst) {
      CMReturnInstance(rslt,inst);
    } else {
      rc.rc=CMPI_RC_ERR_NOT_FOUND;
      rc.msg=CMNewString(_broker,
			 "Failed to extract BIOS Product Information",NULL);
    }
  }else {
    rc.rc=CMPI_RC_ERROR;
    rc.msg=CMNewString(_broker,"Failed to locate BIOS",NULL);
  }

  CMReturnDone(rslt);
  return rc;
}

CMPIStatus Linux_BIOSProductCreateInstance( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * cop, 
  CMPIInstance * ci) {
  CMPIStatus rc = {CMPI_RC_OK, NULL};
  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI CreateInstance()\n", _ClassName ); 
  CMSetStatusWithChars( _broker, &rc, 
			CMPI_RC_ERR_NOT_SUPPORTED, "NOT_SUPPORTED" ); 
  return rc;
}

CMPIStatus Linux_BIOSProductSetInstance( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * cop,
  CMPIInstance * ci, 
  char **properties) {
  CMPIStatus rc = {CMPI_RC_OK, NULL};
  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI SetInstance()\n", _ClassName ); 
  CMSetStatusWithChars( _broker, &rc, 
			CMPI_RC_ERR_NOT_SUPPORTED, "NOT_SUPPORTED" ); 
  return rc;
}

CMPIStatus Linux_BIOSProductDeleteInstance( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * cop) {
  CMPIStatus rc = {CMPI_RC_OK, NULL}; 
  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI DeleteInstance()\n", _ClassName ); 
  CMSetStatusWithChars( _broker, &rc, 
			CMPI_RC_ERR_NOT_SUPPORTED, "NOT_SUPPORTED" ); 
  return rc;
}

CMPIStatus Linux_BIOSProductExecQuery( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * ref, 
  char * lang, 
  char * query) {
  CMPIStatus rc = {CMPI_RC_OK, NULL};
  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI ExecQuery()\n", _ClassName ); 
  CMSetStatusWithChars( _broker, &rc, 
			CMPI_RC_ERR_NOT_SUPPORTED, "NOT_SUPPORTED" ); 
  return rc;
}



/* ---------------------------------------------------------------------------*/
/*                              private methods                               */
/* ---------------------------------------------------------------------------*/
static CMPIObjectPath *makeObjectPath(DMI_BIOSPRODUCT *bp,CMPIObjectPath *op)
{
  CMPIObjectPath * iop=NULL;
  iop = CMNewObjectPath(_broker,CMGetCharPtr(CMGetNameSpace(op,NULL)),_ClassName,NULL);
  if (iop) {
    CMSetNameSpaceFromObjectPath(iop,op);
    CMAddKey(iop,"Name",
	     bp->dmi_Name,CMPI_chars);
    CMAddKey(iop,"IdentifyingNumber",
	     bp->dmi_IdentifyingNumber,CMPI_chars);
    CMAddKey(iop,"Version",
	     bp->dmi_Version,CMPI_chars);
    CMAddKey(iop,"Vendor",
	     bp->dmi_Vendor,CMPI_chars);
  }
  return iop;
}

static CMPIInstance *makeInstance(DMI_BIOSPRODUCT *bp,CMPIObjectPath *op)
{
  CMPIObjectPath * iop=NULL;
  CMPIInstance   * inst=NULL;
  iop = CMNewObjectPath(_broker,CMGetCharPtr(CMGetNameSpace(op,NULL)),_ClassName,NULL);
  if (iop) {
    CMSetNameSpaceFromObjectPath(iop,op);
    inst = CMNewInstance(_broker,iop,NULL);
  }
  if (inst) {
    CMSetProperty(inst,"Name",
		  bp->dmi_Name,CMPI_chars);
    CMSetProperty(inst,"IdentifyingNumber",
		  bp->dmi_IdentifyingNumber,CMPI_chars);
    CMSetProperty(inst,"Version",
		  bp->dmi_Version,CMPI_chars);
    CMSetProperty(inst,"Vendor",
		  bp->dmi_Vendor,CMPI_chars);
  }
  return inst;
}

/* ---------------------------------------------------------------------------*/
/*                              Provider Factory                              */
/* ---------------------------------------------------------------------------*/

CMInstanceMIStub( Linux_BIOSProduct, Linux_BIOSProduct, _broker, cimdmi_init());

/* ---------------------------------------------------------------------------*/
/*                    end of Linux_BIOSProduct                      */
/* ---------------------------------------------------------------------------*/

