/*
 * Linux_BIOSProductBIOSFeatures.c
 *
 * (C) Copyright IBM Corp. 2003
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://oss.software.ibm.com/developerworks/opensource/license-cpl.html
 *
 * Author:       Viktor Mihajlovski <mihajlov@de.ibm.com>
 * Contributors: James Jeffers <jamesjef@us.ibm.com>
 *
 * Interface Type : Common Manageability Programming Interface ( CMPI )
 *
 * Description: Association between BIOS Product and Feature.
 * 
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "cmpidt.h"
#include "cmpift.h"
#include "cmpimacs.h"


unsigned char CMPI_true=1;
unsigned char CMPI_false=0;

CMPIBroker * _broker;

/* ---------------------------------------------------------------------------*/
/* private declarations                                                       */
#ifdef DEBUG
    int _debug = 1;
#else
    int _debug = 0;
#endif

static char * _ClassName     = "Linux_BIOSProductBIOSFeatures";
static char * _RefLeft       = "Product";
static char * _RefRight      = "Component";
static char * _RefLeftClass  = "Linux_BIOSProduct";
static char * _RefRightClass = "Linux_BIOSFeature";

static CMPIObjectPath *makeObjectPath(CMPIObjectPath *opLeft,
				      CMPIObjectPath *opRight);
static CMPIInstance *makeInstance(CMPIObjectPath *opLeft,
				  CMPIObjectPath *opRight);

/* ---------------------------------------------------------------------------*/


/* ---------------------------------------------------------------------------*/
/*                      Instance Provider Interface                           */
/* ---------------------------------------------------------------------------*/


CMPIStatus Linux_BIOSProductBIOSFeaturesCleanup( CMPIInstanceMI * mi, 
  CMPIContext * ctx) {
  CMReturn(CMPI_RC_OK);
}

CMPIStatus Linux_BIOSProductBIOSFeaturesEnumInstanceNames( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * ref) { 
  CMPIStatus rc = {CMPI_RC_OK, NULL};
  CMPIObjectPath  *copLeft, *copRight;
  CMPIData valLeft, valRight;
  CMPIEnumeration *enmLeft, *enmRight;
  
  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI EnumInstanceNames()\n", _ClassName );
 
  copLeft = CMNewObjectPath(_broker,"",_RefLeftClass,NULL);
  copRight = CMNewObjectPath(_broker,"",_RefRightClass,NULL);
  if (copLeft && copRight) {
    CMSetNameSpaceFromObjectPath(copLeft,ref);
    CMSetNameSpaceFromObjectPath(copRight,ref);
    enmLeft = CBEnumInstanceNames(_broker,ctx,copLeft,&rc);
    enmRight = CBEnumInstanceNames(_broker,ctx,copRight,&rc);
    valLeft = CMGetNext(enmLeft,NULL);
    valRight = CMGetNext(enmRight,NULL);
    if (valLeft.value.ref && valRight.value.ref) {
      CMReturnObjectPath(rslt,makeObjectPath(valLeft.value.ref,
					     valRight.value.ref));
    } else {
      rc.msg = CMNewString(_broker,"Failed to get REF variables.",NULL);
      rc.rc = CMPI_RC_ERR_NOT_FOUND;
    }
  } else {
    rc.msg = CMNewString(_broker,"Failed to get REF class definitions.",NULL);
    rc.rc = CMPI_RC_ERR_FAILED;
  }
  
  CMReturnDone( rslt );
  return rc;
}

CMPIStatus Linux_BIOSProductBIOSFeaturesEnumInstances( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * ref, 
  char ** properties) { 
  CMPIStatus rc = {CMPI_RC_OK, NULL};
  CMPIObjectPath  *copLeft, *copRight;
  CMPIData valLeft, valRight;
  CMPIEnumeration *enmLeft, *enmRight;

  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI EnumInstances()\n", _ClassName );

  copLeft = CMNewObjectPath(_broker,"",_RefLeftClass,NULL);
  copRight = CMNewObjectPath(_broker,"",_RefRightClass,NULL);
  if (copLeft && copRight) {
    CMSetNameSpaceFromObjectPath(copLeft,ref);
    CMSetNameSpaceFromObjectPath(copRight,ref);
    enmLeft = CBEnumInstanceNames(_broker,ctx,copLeft,&rc);
    enmRight = CBEnumInstanceNames(_broker,ctx,copRight,&rc);
    valLeft = CMGetNext(enmLeft,NULL);
    valRight = CMGetNext(enmRight,NULL);
    if (valLeft.value.ref && valRight.value.ref) {
      CMReturnInstance(rslt,makeInstance(valLeft.value.ref,
					 valRight.value.ref));
    } else {
      rc.msg = CMNewString(_broker,"Failed to get REF variables.",NULL);
      rc.rc = CMPI_RC_ERR_NOT_FOUND;
    }
  } else {
    rc.msg = CMNewString(_broker,"Failed to get REF class definitions.",NULL);
    rc.rc = CMPI_RC_ERR_FAILED;
  }

  CMReturnDone( rslt );
  return rc;
}

CMPIStatus Linux_BIOSProductBIOSFeaturesGetInstance( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * cop, 
  char ** properties) {
  CMPIStatus rc = {CMPI_RC_OK, NULL};
  CMPIData valLeft, valRight;
  CMPIInstance *instLeft, *instRight;
  
  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI GetInstance()\n", _ClassName );

  valLeft = CMGetKey(cop,_RefLeft,NULL);
  valRight = CMGetKey(cop,_RefRight,NULL);
  if ( valLeft.value.ref && valRight.value.ref) {
    /* check for existence */
    instLeft = CBGetInstance(_broker,ctx,valLeft.value.ref,NULL,&rc);
    instRight = CBGetInstance(_broker,ctx,valRight.value.ref,NULL,&rc);
    if (instLeft && instRight) {
      CMReturnInstance(rslt,makeInstance(valLeft.value.ref,
					 valRight.value.ref));
    } else {
      rc.msg = CMNewString(_broker,"Failed to get REF variables.",NULL);
      rc.rc = CMPI_RC_ERR_NOT_FOUND;
    } 
  } else {
    rc.msg = CMNewString(_broker,"Failed to get REF class definitions.",NULL);
    rc.rc = CMPI_RC_ERR_FAILED;
  }
  
  CMReturnDone( rslt );
  return rc;

}

CMPIStatus Linux_BIOSProductBIOSFeaturesCreateInstance( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * cop, 
  CMPIInstance * ci) {
  CMPIStatus rc = {CMPI_RC_OK, NULL};
  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI CreateInstance()\n", _ClassName ); 
  CMSetStatusWithChars( _broker, &rc, 
			CMPI_RC_ERR_NOT_SUPPORTED, "NOT_SUPPORTED" ); 
  return rc;
}

CMPIStatus Linux_BIOSProductBIOSFeaturesSetInstance( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * cop,
  CMPIInstance * ci, 
  char **properties) {
  CMPIStatus rc = {CMPI_RC_OK, NULL};
  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI SetInstance()\n", _ClassName ); 
  CMSetStatusWithChars( _broker, &rc, 
			CMPI_RC_ERR_NOT_SUPPORTED, "NOT_SUPPORTED" ); 
  return rc;
}

CMPIStatus Linux_BIOSProductBIOSFeaturesDeleteInstance( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * cop) {
  CMPIStatus rc = {CMPI_RC_OK, NULL}; 
  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI DeleteInstance()\n", _ClassName ); 
  CMSetStatusWithChars( _broker, &rc, 
			CMPI_RC_ERR_NOT_SUPPORTED, "NOT_SUPPORTED" ); 
  return rc;
}

CMPIStatus Linux_BIOSProductBIOSFeaturesExecQuery( CMPIInstanceMI * mi, 
  CMPIContext * ctx, 
  CMPIResult * rslt, 
  CMPIObjectPath * ref, 
  char * lang, 
  char * query) {
  CMPIStatus rc = {CMPI_RC_OK, NULL};
  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI ExecQuery()\n", _ClassName ); 
  CMSetStatusWithChars( _broker, &rc, 
			CMPI_RC_ERR_NOT_SUPPORTED, "NOT_SUPPORTED" ); 
  return rc;
}


/* ---------------------------------------------------------------------------*/
/*                    Associator Provider Interface                           */
/* ---------------------------------------------------------------------------*/


CMPIStatus Linux_BIOSProductBIOSFeaturesAssociationCleanup( CMPIAssociationMI * mi,
  CMPIContext * ctx) {
  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI AssociationCleanup()\n", _ClassName ); 
  CMReturn(CMPI_RC_OK);
}

/* 
 * The intention of associations is to show the relations between different
 * classes and their instances. Therefore an association has two properties. 
 * Each one representing a reference to a certain instance of the specified
 * class. We can say, that an association has a left and the right "end". 
 * 
 * Linux_BIOSProductBIOSFeatures : 
 *    < role >   -> < class >
 *    "_RefLeft"  -> "_RefLeftClass" 
 *    "_RefRight" -> "_RefRightClass"
 *
 */

/*
 * general API information
 *
 * cop :
 *    Specifies the CMPIObjectPath to the CMPIInstance of the known end of 
 *    the association. Its absolutelly necessary to define this anchor point,
 *    from where the evaluation of the association starts.
 * assocPath : 
 *    If not NULL, it contains the name of the association the caller is 
 *    looking for. The provider should only return values, when the assocPath 
 *    contains the name(s) of the association(s) he is responsible for !
 * role :
 *    The caller can specify that the source instance ( cop ) has to play 
 *    a certain <role> in the association. The <role> is specified by the
 *    association definition (see above). That means, role has to contain
 *    the same value as the <role> the source instance plays in this assoc.
 *    If this requirement is not true, the provider returns nothing.
 */

/*
 * specification of associators() and associatorNames()
 *
 * These methods return CMPIInstance ( in the case of associators() ) or 
 * CMPIObjectPath ( in the case of associatorNames() ) object(s) of the 
 * opposite end of the association.
 *
 * resultRole :
 *    The caller can specify that the target instance(s) has/have to play 
 *    a certain <role> in the association. The <role> is specified by the
 *    association definition (see above). That means, resultRole has to 
 *    contain the same value as the <role> the target instance(s) plays
 *    in this assoc. If this requirement is not true, the provider returns 
 *    nothing.
 * resultClass :
 *    The caller can specify that the target instance(s) has/have to be 
 *    instances of a certain <class>. The <class> is specified by the
 *    association definition (see above). That means, resultClass has to 
 *    contain the same value as the <class> of the target instance(s).
 *    If this requirement is not true, the provider returns nothing.
 */

CMPIStatus Linux_BIOSProductBIOSFeaturesAssociators( CMPIAssociationMI * mi,
  CMPIContext * ctx,
  CMPIResult * rslt,
  CMPIObjectPath * cop,
  char * assocClass,
  char * resultClass,
  char * role,
  char * resultRole,
  char ** propertyList ) {
  CMPIStatus       rc = {CMPI_RC_OK, NULL};
  CMPIObjectPath * op = NULL;

  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI Associators()\n", _ClassName ); 
    
  if( assocClass ) {
    op = CMNewObjectPath( _broker, CMGetCharPtr(CMGetNameSpace(cop,&rc)),
			  _ClassName, &rc );
  }

  if( ( assocClass==NULL ) || ( CMClassPathIsA(_broker,op,assocClass,&rc) == 1 ) ) {
    if (CMClassPathIsA(_broker,cop,_RefLeftClass,&rc)) {
      /* source is left hand side - retrieve right hand side */
      CMPIObjectPath *copRight = 
	CMNewObjectPath(_broker,CMGetCharPtr(CMGetNameSpace(cop,&rc)),
			_RefRightClass, &rc);
      if (copRight) {
	CMPIEnumeration *enm = CBEnumInstances(_broker,ctx,copRight,NULL,&rc);
	if (enm) {
	  while(CMHasNext(enm,&rc)) {
	    CMPIData data = CMGetNext(enm,&rc);
	    CMReturnInstance(rslt,data.value.inst);
	  }
	}
      } else {
	rc.rc = CMPI_RC_ERROR;
	rc.msg = CMNewString(_broker,"Failed to build right object path",NULL);
      }
    } else if (CMClassPathIsA(_broker,cop,_RefRightClass,&rc)) {
      /* source is left hand side - retrieve right hand side */
      CMPIObjectPath *copLeft = 
	CMNewObjectPath(_broker,CMGetCharPtr(CMGetNameSpace(cop,&rc)),
			_RefLeftClass, &rc);
      if (copLeft) {
	CMPIEnumeration *enm = CBEnumInstances(_broker,ctx,copLeft,NULL,&rc);
	if (enm) {
	  while(CMHasNext(enm,&rc)) {
	    CMPIData data = CMGetNext(enm,&rc);
	    CMReturnInstance(rslt,data.value.inst);
	  }
	}
      } else {
	rc.rc = CMPI_RC_ERROR;
	rc.msg = CMNewString(_broker,"Failed to build left object path",NULL);
      }
    }
  }

  CMReturnDone( rslt );
  return rc;
}

CMPIStatus Linux_BIOSProductBIOSFeaturesAssociatorNames( CMPIAssociationMI * mi,
  CMPIContext * ctx,
  CMPIResult * rslt,
  CMPIObjectPath * cop,
  char * assocClass,
  char * resultClass,
  char * role,
  char * resultRole) {
  CMPIStatus       rc = {CMPI_RC_OK, NULL};
  CMPIObjectPath * op = NULL;

  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI AssociatorNames()\n", _ClassName ); 
    
  if( assocClass ) {
    op = CMNewObjectPath( _broker, CMGetCharPtr(CMGetNameSpace(cop,&rc)),
			  _ClassName, &rc );
  }

  if( ( assocClass==NULL ) || ( CMClassPathIsA(_broker,op,assocClass,&rc) == 1 ) ) {
    if (CMClassPathIsA(_broker,cop,_RefLeftClass,&rc)) {
      /* source is left hand side - retrieve right hand side */
      CMPIObjectPath *copRight = 
	CMNewObjectPath(_broker,CMGetCharPtr(CMGetNameSpace(cop,&rc)),
			_RefRightClass, &rc);
      if (copRight) {
	CMPIEnumeration *enm = 
	  CBEnumInstanceNames(_broker,ctx,copRight,&rc);
	if (enm) {
	  while(CMHasNext(enm,&rc)) {
	    CMPIData data = CMGetNext(enm,&rc);
	    CMReturnObjectPath(rslt,data.value.ref);
	  }
	}
      } else {
	rc.rc = CMPI_RC_ERROR;
	rc.msg = CMNewString(_broker,"Failed to build right object path",NULL);
      }
    } else if (CMClassPathIsA(_broker,cop,_RefRightClass,&rc)) {
      /* source is right hand side - retrieve left hand side */
      CMPIObjectPath *copLeft = 
	CMNewObjectPath(_broker,CMGetCharPtr(CMGetNameSpace(cop,&rc)),
			_RefLeftClass, &rc);
      if (copLeft) {
	CMPIEnumeration *enm = 
	  CBEnumInstanceNames(_broker,ctx,copLeft,&rc);
	if (enm) {
	  while(CMHasNext(enm,&rc)) {
	    CMPIData data = CMGetNext(enm,&rc);
	    CMReturnObjectPath(rslt,data.value.ref);
	  }
	}
      } else {
	rc.rc = CMPI_RC_ERROR;
	rc.msg = CMNewString(_broker,"Failed to build left object path",NULL);
      }
    }
  }

  CMReturnDone( rslt );
  CMReturn(CMPI_RC_OK);
}


/*
 * specification of references() and referenceNames()
 *
 * These methods return CMPIInstance ( in the case of references() ) or 
 * CMPIObjectPath ( in the case of referenceNames() ) object(s) of th 
 * association itself.
 */

CMPIStatus Linux_BIOSProductBIOSFeaturesReferences( CMPIAssociationMI * mi,
  CMPIContext * ctx,
  CMPIResult * rslt,
  CMPIObjectPath * cop,
  char * assocClass,
  char * role,
  char ** propertyList ) {
  CMPIStatus       rc = {CMPI_RC_OK, NULL};
  CMPIObjectPath * op = NULL;

  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI References()\n", _ClassName ); 
    
  op = CMNewObjectPath( _broker, CMGetCharPtr(CMGetNameSpace(cop,&rc)),
			  _ClassName, &rc );
  
  if( ( assocClass==NULL ) || ( CMClassPathIsA(_broker,op,assocClass,&rc) == 1 ) ) {
    if (CMClassPathIsA(_broker,cop,_RefLeftClass,&rc) || 
	CMClassPathIsA(_broker,cop,_RefRightClass,&rc) ) {
      return Linux_BIOSProductBIOSFeaturesEnumInstances((CMPIInstanceMI*)mi,
							ctx,rslt,op,
							propertyList);
    }
    
  }
  
  CMReturnDone( rslt );
  CMReturn(CMPI_RC_OK);
}

CMPIStatus Linux_BIOSProductBIOSFeaturesReferenceNames( CMPIAssociationMI * mi,
  CMPIContext * ctx,
  CMPIResult * rslt,
  CMPIObjectPath * cop,
  char * assocClass,
  char * role) {
  CMPIStatus       rc = {CMPI_RC_OK, NULL};
  CMPIObjectPath * op = NULL;

  if( _debug )
    fprintf( stderr, "--- %s.c : CMPI ReferenceNames()\n", _ClassName ); 
    
  op = CMNewObjectPath( _broker, CMGetCharPtr(CMGetNameSpace(cop,&rc)),
			_ClassName, &rc );
  if( ( assocClass==NULL ) || ( CMClassPathIsA(_broker,op,assocClass,&rc) == 1 ) ) {
    if (CMClassPathIsA(_broker,cop,_RefLeftClass,&rc) || 
	CMClassPathIsA(_broker,cop,_RefRightClass,&rc) ) {
      return Linux_BIOSProductBIOSFeaturesEnumInstanceNames((CMPIInstanceMI*)mi,
							    ctx,rslt,op);
    }
  }

  CMReturnDone( rslt );
  CMReturn(CMPI_RC_OK);
}



/* ---------------------------------------------------------------------------*/
/*                              private methods                               */
/* ---------------------------------------------------------------------------*/
static CMPIObjectPath *makeObjectPath(CMPIObjectPath *opLeft,
				      CMPIObjectPath *opRight)
{
  CMPIObjectPath *opAss = CMNewObjectPath(_broker,"",_ClassName,NULL);
  if (opAss) {
    CMSetNameSpaceFromObjectPath(opAss,opLeft);
    CMAddKey(opAss,_RefLeft,&opLeft,CMPI_ref);
    CMAddKey(opAss,_RefRight,&opRight,CMPI_ref);
  }
  return opAss;
}

static CMPIInstance *makeInstance(CMPIObjectPath *opLeft,
				  CMPIObjectPath *opRight)
{
  CMPIObjectPath *opAss = CMNewObjectPath(_broker,"",_ClassName,NULL);
  CMPIInstance   *inst = NULL;
  if (opAss) {
    CMSetNameSpaceFromObjectPath(opAss,opLeft);
    inst = CMNewInstance(_broker,opAss,NULL);
    if (inst) {
      CMSetProperty(inst,_RefLeft,&opLeft,CMPI_ref);
      CMSetProperty(inst,_RefRight,&opRight,CMPI_ref);
    }
  }
  return inst;
}

/* ---------------------------------------------------------------------------*/
/*                              Provider Factory                              */
/* ---------------------------------------------------------------------------*/

CMInstanceMIStub( Linux_BIOSProductBIOSFeatures, Linux_BIOSProductBIOSFeatures, _broker, CMNoHook);

CMAssociationMIStub( Linux_BIOSProductBIOSFeatures, Linux_BIOSProductBIOSFeatures, _broker, CMNoHook);

/* ---------------------------------------------------------------------------*/
/*                    end of Linux_BIOSProductBIOSFeatures                      */
/* ---------------------------------------------------------------------------*/

